<?php

require_once DIR_FS_CATALOG . DIR_WS_CLASSES . 'vendors/gratifypay/plugin-php-sdk/autoload.php';

use GratifyPay\PhpSdk\Client as GratifyPayClient;
use GratifyPay\PhpSdk\Request\Item;
use GratifyPay\PhpSdk\Request\OrderInitialize;
use GratifyPay\PhpSdk\Request\ShippingAddress;

class GratifyPayService
{
    const STORE_PLATFORM = 'ZENCART';

    /**
     * @var GratifyPay\PhpSdk\Client
     */
    protected $gratifyClient;

    public function __construct()
    {
        $this->gratifyClient = new GratifyPayClient(
            MODULE_PAYMENT_GRATIFYPAY_MERCHANT_ID,
            MODULE_PAYMENT_GRATIFYPAY_MERCHANT_SECRET_KEY,
            true
        );
    }

    public function getPaymentPeriod($price, $page)
    {
        if ($this->isAvailable($price)) {
            require_once(DIR_WS_CLASSES . 'currencies.php');
            $currencies = new currencies();

            list($numPayments, $firstPayment) = $this->getFirstPaymentByPrice($price);

            $search = ['{NUM_PAYMENTS}', '{AMOUNT}', '{GRATIFY_LOGO}'];
            $replace = [
                $numPayments,
                $currencies->display_price($firstPayment, 0),
                '<img style="margin-top:-3px;vertical-align:middle;display:inline-block;border:none;"
            src="https://assets.dev.gratifypay.com/img/gratify-badge-blue.svg" width="80" height="30" alt="Gratify Pay" />'
            ];
            $phrase = '';
            switch ($page) {
                case 'category':
                    if (MODULE_PAYMENT_GRATIFYPAY_CATEGORY_PAGE == 'True') {
                        $phrase = MODULE_PAYMENT_GRATIFYPAY_CATEGORY_PAGE_PHRASE;
                    }
                    break;
                case 'product':
                    if (MODULE_PAYMENT_GRATIFYPAY_PRODUCT_PAGE == 'True') {
                        $phrase = MODULE_PAYMENT_GRATIFYPAY_PRODUCT_PAGE_PHRASE;
                    }
                    break;
                case 'cart':
                    if (MODULE_PAYMENT_GRATIFYPAY_CART_PAGE == 'True') {
                        $phrase = MODULE_PAYMENT_GRATIFYPAY_CART_PAGE_PHRASE;
                    }
                    break;
            }

            if ($phrase) {
                $phrase = str_replace($search, $replace, $phrase);
            }

            return $phrase;
        }
    }

    public function isAvailable($price)
    {
        if ($price) {
            static $merchant;
            try {
                if (empty($merchant)) {
                    $merchant = $this->gratifyClient->getMerchant();
                }
            } catch (\Exception $e) {
                return false;
            }
            if ($price >= $merchant->getMinAmount() && $price <= $merchant->getMaxAmount()) {
                return true;
            }
        }

        return false;
    }

    public function getFirstPaymentByPrice($price)
    {
        $paymentSchedule = $this->gratifyClient->getPaymentsSchedule($price);
        $numPayments = count($paymentSchedule->getSchedules());
        $firstPayment = isset($paymentSchedule->getSchedules()[0]) ? $paymentSchedule->getSchedules()[0]->getAmount() : null;

        return [$numPayments, $firstPayment];
    }

    public function getPaymentShedule($price)
    {
        if (!$this->isAvailable($price)) {
            return null;
        }

        $schedules = [
            'schedules' => [],
            'meta' => [
                'numPayments' => 0,
                'totalAmount' => 0,
            ],
        ];
        if ($price) {
            $paymentSchedule = $this->gratifyClient->getPaymentsSchedule($price, true);

            foreach ($paymentSchedule->getSchedules() as $schedule) {
                $schedules['schedules'][] = [
                    'date' => $schedule->getDate(),
                    'amount' => $schedule->getAmount(),
                    'dueFromNow' => $schedule->getDueFromNow(),
                    'paymentNumber' => $schedule->getPaymentNumber(),
                    'totalAmount' => $schedule->getTotalAmount(),
                ];
            }

            $schedules['meta'] = [
                'numPayments' => $paymentSchedule->getMeta()->getNumPayments(),
                'totalAmount' => $paymentSchedule->getMeta()->getTotalAmount(),
            ];
        }

        return $schedules;
    }

    public function orderInitialize()
    {
        global $db;

        $sql = "SELECT entry_firstname AS firstname, entry_lastname AS lastname,
                   entry_company AS company, entry_street_address AS street_address,
                   entry_suburb AS suburb, entry_city AS city, entry_postcode AS postcode,
                   entry_state AS state, entry_zone_id AS zone_id,
                   entry_country_id AS country_id
            FROM " . TABLE_ADDRESS_BOOK . "
            WHERE customers_id = " . (int)$_SESSION['customer_id'] . "
            AND address_book_id = " . (int)$_SESSION['sendto'];

        $address = $db->Execute($sql);
        $address = $address->fields;

        $shippingAddress = new ShippingAddress();
        $shippingAddress->setCity($address['city'] ?? '')
            ->setCountry(zen_get_country_name($address['country_id']) ?? '')
            ->setLine1($address['street_address'] ?? '')
            ->setState($address['state'] ?? '')
            ->setPostcode($address['postcode'] ?? '');

        $order = new OrderInitialize();
        $tax = 0;
        foreach ($_SESSION['cart']->get_products() as $product) {
            $total_price = (float)$product['final_price'] * (int)$product['quantity'];
            $item = new Item();
            $item->setTitle($product['name'])
                ->setDescription('')
                ->setImageURL(HTTPS_SERVER . '/images/' . $product['image'])
                ->setId($product['id'])
                ->setPrice((float)$product['final_price'])
                ->setQuantity((int)$product['quantity'])
                ->setTotalPrice($total_price);

            $order->setItem($item);

            $tax += zen_calculate_tax($total_price, zen_get_tax_rate($product['products_tax_class_id']));
        }

        $order->setStorePlatform(self::STORE_PLATFORM)
            ->setMerchantReference($_SESSION['cart']->cartID)
            ->setCallbackURL(zen_href_link("gratifypay_callback", 'cart_id=' . (int)$_SESSION['cart']->cartID))
            ->setFailureURL(zen_href_link("gratifypay_failure", 'cart_id=' . (int)$_SESSION['cart']->cartID))
            ->setReturnURL(zen_href_link("gratifypay_success", 'cart_id=' . (int)$_SESSION['cart']->cartID))
            ->setPriceSubTotal($_SESSION['cart']->show_total())
            ->setTaxesTotal($tax)
            ->setShippingMethod(OrderInitialize::SHIPPING_METHOD)
            ->setShippingAddress($shippingAddress)
            ->setShippingTotal($_SESSION['shipping']['cost'] ?? 0);

        $gratifyPayLink = null;
        if ($result = $this->gratifyClient->orderInitialize($order)) {
            $gratifyPayLink = $this->gratifyClient->getRedirectLink($result->getOrderMapId());
        }

        return $gratifyPayLink;
    }

    public function confirmationOrder($orderId, $merchantReference)
    {
        return $this->gratifyClient->confirmationOrder($orderId, $merchantReference, self::STORE_PLATFORM);
    }

    public function getGratifyPhrase($page, $numPayments, $amount)
    {
        $search = ['{NUM_PAYMENTS}', '{AMOUNT}', '{GRATIFY_LOGO}'];
        $replace = [
            $numPayments,
            $amount,
            '<img style="margin-top:-3px;vertical-align:middle;display:inline-block;border:none;"
            src="https://assets.dev.gratifypay.com/img/gratify-badge-blue.svg" width="80" height="30" alt="Gratify Pay" />'
        ];
        $phrase = '';
        switch ($page) {
            case 'category':
                $phrase = MODULE_PAYMENT_GRATIFYPAY_CATEGORY_PAGE_PHRASE;
                break;
            case 'product':
                $phrase = MODULE_PAYMENT_GRATIFYPAY_PRODUCT_PAGE_PHRASE;
                break;
            case 'cart':
                $phrase = MODULE_PAYMENT_GRATIFYPAY_CART_PAGE_PHRASE;
                break;
        }

        return str_replace($search, $replace, $phrase);
    }
}
